# Generated by Django 5.0.9 on 2024-10-03 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sop_phone', '0002_phonemaintainer_latitude_phonemaintainer_longitude_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='phonemaintainer',
            name='shipping_address',
            field=models.CharField(blank=True, max_length=200),
        ),
    ]
