# Generated by Django 5.0.9 on 2024-10-03 08:51

import timezone_field.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sop_phone', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='phonemaintainer',
            name='latitude',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=8, null=True),
        ),
        migrations.AddField(
            model_name='phonemaintainer',
            name='longitude',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='phonemaintainer',
            name='physical_address',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='phonemaintainer',
            name='time_zone',
            field=timezone_field.fields.TimeZoneField(blank=True),
        ),
    ]
